# ::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::: #
# Date: 08.09.2016                                                            #
# Author: Ole-Johan Skrede                                                    #
#                                                                             #
# Solution proposal as part of the exercise program in                        #
# INF4300 - Digital image analysis at the University of Oslo                  #
#                                                                             #
# ::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::: #
"""
Solution proposal for exercise 3, task 2.
"""

# pylint: disable=expression-not-assigned
# pylint: disable=bad-indentation
# pylint: disable=redefined-outer-name

import os
import argparse

import cv2
import matplotlib.pyplot as plt

import hough_transform

def main():
  """Use Hough transform to detect lines in an image"""

  parser = argparse.ArgumentParser()
  parser.add_argument('-d', '--detect', default='lines',
                      help='{lines}')
  parser.add_argument('-at', '--acc_threshold', type=int, default=100,
                      help=('Accumulator threshold. Number of counts a point'
                            'need in order to be a candidate solution.'))
  parser.add_argument('-ot', '--obj_threshold', type=int, default=100,
                      help=('Object threshold. Number of actual foreground'
                            'pixels a candidate solution need in order to be'
                            'an actual solution.'))
  parser.add_argument('-mi', '--max_iter', type=int, default=1000,
                      help='Maximum number of iterations before search termination')
  parser.add_argument('-i', '--in_filepath', required=True,
                      help='Full filepath of input image (grayscale image)')
  parser.add_argument('-o', '--result_dir', default='./results/task02',
                      help='Output filepath (is created if it not exists)')
  args = parser.parse_args()

  print('='*80)
  print('Solution to weekly exercises in INF4300')
  print('Exercise 3')
  print('Task 2')
  print('-'*80)

  # Run program
  rand_hough = hough_transform.RandomHoughTransform(args.in_filepath, args.detect,
                                                    args.acc_threshold, args.obj_threshold)
  rand_hough.run(args.max_iter)
  rand_hough.draw_solution_image()

  # Plot results
  plt.figure(1)
  plt.subplot(1, 3, 1)
  plt.imshow(cv2.cvtColor(rand_hough.image, cv2.COLOR_BGR2RGB))
  plt.title('Original'), plt.xticks([]), plt.yticks([])
  plt.subplot(1, 3, 2)
  plt.imshow(rand_hough.edges, cmap='gray')
  plt.title('Edges'), plt.xticks([]), plt.yticks([])
  plt.subplot(1, 3, 3)
  plt.imshow(cv2.cvtColor(rand_hough.lined_image, cv2.COLOR_BGR2RGB))
  plt.title('Original with lines'), plt.xticks([]), plt.yticks([])

  # Write results to file
  if not os.path.exists(args.result_dir):
    print("Creating result folder: {}".format(args.result_dir))
    os.makedirs(args.result_dir)
  in_name = os.path.splitext(os.path.basename(args.in_filepath))[0]
  cv2.imwrite(os.path.join(args.result_dir, in_name + '_edges.png'), rand_hough.edges)
  cv2.imwrite(os.path.join(args.result_dir, in_name + '_lined.png'), rand_hough.lined_image)

  plt.show()

if __name__ == "__main__":
  main()
